/*
 *  ParticleSystem.cpp
 *  GLUT-PS
 *
 *  Created by Serato on 17/09/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "ParticleSystem.h"
#include "shaderdev.h"

#include "glincludes.h"


#define PARTICLE_TEXTURE_FILENAME "images/rain.png"
#define PARTICLE_EMISSION 100
#define PARTICLE_RADIUS 0.1
#define PARTICLE_STACKS 16
#define PARTICLE_MAX 100000
#define PARTICLE_SPREAD 2
#define PARTICLE_SPEED 3

// Set up the relevant colour values for the particles
const GLfloat diffuse[] = {0.2f,0.8f,1.0f,0.1f};
const GLfloat ambient[] = {0.2f,0.8f,1.0f,0.1f};
const GLfloat specular[] = {1.0f,1.0f,1.0f,0.1f};
const GLfloat shiny[] = {10.0f};


ParticleSystem::ParticleSystem(){
	systemTime = 0.0;
	posArrayIndex = 0;
	particles = new Particle[PARTICLE_MAX];
	drawMethod = 1;
	
}

ParticleSystem::~ParticleSystem(){

}

void ParticleSystem::setDrawMethod(int type){
	drawMethod = type;
}
	
void ParticleSystem::init(){
	// initalize all particles to be non-existent
	Particle * current;
	for(int i=0; i< PARTICLE_MAX; i++){
		current = &particles[i];
		current->X = new float[6];
		current->X[0] = 0;
		current->X[1] = 0;
		current->X[2] = 0;
		current->X[3] = 0;
		current->X[4] = 0;
		current->X[5] = 0;
		current->DX = new float[6];
		current->DX[0] = 0;
		current->DX[1] = 0;
		current->DX[2] = 0;
		current->DX[3] = 0;
		current->DX[4] = 0;
		current->DX[5] = 0;
		current->radius = 0;
		current->time = 0;
	}

	particleTexture = registerImage(PARTICLE_TEXTURE_FILENAME);
}

void ParticleSystem::draw(){
	Particle * current;
	glShadeModel(GL_SMOOTH);
	
	glPushMatrix();
	switch(drawMethod){
		case 1: // geometry
			glMaterialfv(GL_FRONT,GL_DIFFUSE, diffuse);
			glMaterialfv(GL_FRONT,GL_AMBIENT, ambient);
			glMaterialfv(GL_FRONT,GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT,GL_SHININESS, shiny);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			break;
		case 2:
			glColor3f(1,1,1);
			glEnable(GL_TEXTURE_2D);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glBindTexture(GL_TEXTURE_2D, particleTexture);
			break;
		case 3:
			glEnable(GL_TEXTURE_2D);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glBindTexture(GL_TEXTURE_2D, particleTexture);
			glTexEnvf(GL_POINT_SPRITE,GL_COORD_REPLACE,GL_TRUE);
			glEnable(GL_POINT_SPRITE);
			glColor3f(1,1,1);
			glPointSize(12.0);
			float quadratic[] =  { 1.0f, 0.0f, 0.01f };
			glPointParameterfv( GL_POINT_DISTANCE_ATTENUATION, quadratic );
			glPointParameterf( GL_POINT_FADE_THRESHOLD_SIZE, 1.0f );
			glPointParameterf( GL_POINT_SIZE_MIN, 1.0f );
			glPointParameterf( GL_POINT_SIZE_MAX, 32.0f ); 	
			glPointParameterf(GL_POINT_SPRITE_COORD_ORIGIN,GL_LOWER_LEFT);
			glBegin(GL_POINTS);
			break;
	}
	GLfloat transform[4][4];
	
	// Draw each particle in turn
	for(int i=0; i < PARTICLE_MAX; i++){
		current = &particles[i];
		// If particle is valid
		if(current->radius != 0){
			switch(drawMethod){
				case 1: // geometry
					glTranslatef(current->X[0], current->X[1], current->X[2]);
					glutSolidSphere(current->radius, PARTICLE_STACKS, PARTICLE_STACKS);
					glTranslatef(-current->X[0], -current->X[1], -current->X[2]);
					break;
				case 2: // billboards
					glPushMatrix();
					glTranslatef(current->X[0], current->X[1], current->X[2]);
					glGetFloatv(GL_MODELVIEW_MATRIX, (GLfloat *)transform);
					for(int i=0;i<3;i++)
					{
						for(int j=0;j<3;j++)
							transform[i][j]=0.0;
						transform[i][i]=1.0;
					}
					glLoadMatrixf((GLfloat *)&transform);
					glBegin(GL_TRIANGLE_STRIP);
					glTexCoord2f(0,0);
					glVertex3f( -PARTICLE_RADIUS, -PARTICLE_RADIUS, 0);
					glTexCoord2f(0,1);
					glVertex3f( - PARTICLE_RADIUS, + PARTICLE_RADIUS, 0);
					glTexCoord2f(1,0);
					glVertex3f( + PARTICLE_RADIUS, - PARTICLE_RADIUS, 0);
					glTexCoord2f(1,1);
					glVertex3f( + PARTICLE_RADIUS, + PARTICLE_RADIUS, 0);
					glEnd();
					glPopMatrix();
					break;
				case 3: // point sprites
					glVertex3f(current->X[0], current->X[1], current->X[2]);
					break;
				default:
					break;
			}
		}
	}
	switch(drawMethod){
		case 1: // geometry
			glDisable(GL_BLEND);
			break;
		case 2:
			glDisable(GL_TEXTURE_2D);
			glDisable(GL_BLEND);
			break;
		case 3:
			glEnd();
			glDisable(GL_POINT_SPRITE);
			glDisable(GL_TEXTURE_2D);
			glDisable(GL_BLEND);
			break;
	}
	glPopMatrix();

}

void ParticleSystem::update(double time){
	systemTime += time;
	
	int i;
	Particle * current;
	
	// EMISSION
	float currentTime = 0;
	//Based on the flow rate specified create new particles - if particles are created faster than destroyed then they will be overwritten
	for ( i = 0; i < PARTICLE_EMISSION; i++){
		current = &particles[posArrayIndex];
		current->X[0] = 0;
		current->X[1] = 0;
		current->X[2] = 5;
		current->X[3] = 2*genrand2()*PARTICLE_SPREAD - PARTICLE_SPREAD;
		current->X[4] = 2*genrand2()*PARTICLE_SPREAD - PARTICLE_SPREAD;
		current->X[5] = PARTICLE_SPEED + 0.1*genrand2()*PARTICLE_SPREAD;
		current->DX[0] = current->X[3];
		current->DX[1] = current->X[4];
		current->DX[2] = current->X[4];
		current->DX[3] = 0;
		current->DX[4] = 0;
		current->DX[5] = 0;
		current->radius = PARTICLE_RADIUS;
		current->time = currentTime;
		posArrayIndex++;
		if(posArrayIndex >= PARTICLE_MAX)
			posArrayIndex = 0;
	}
	
	// SOLVER UPDATE
	// evaluate each particle (using the midpoint method in PartSystem.cpp) and check that they are not below the waterline
	for(int i=0; i < PARTICLE_MAX; i++){
		current = &particles[i];
		if(current->radius != 0){
			midPoint(current, time/1000.0);
			// If below the water
			if(current->X[2] < 0.0){
				//destroy
				current->radius = 0;
			}
		}
	}
	
}
