// bump map shader

varying vec3 lightDir;    // interpolated surface local coordinate light direction 
varying vec3 viewDir;     // interpolated surface local coordinate view direction

uniform sampler2D colourMap;
uniform sampler2D normalMap;
void main (void)
{
	vec3 lightVec = normalize(lightDir);
	vec3 viewVec = normalize(viewDir);

	vec4 intensity = gl_LightSource[0].ambient;
	// retrieve normal vector from normal map
	vec3 norm = normalize(texture2D(normalMap, gl_TexCoord[0].st).xyz * 2.0 - 1.0);
	
	// compute diffuse component
	intensity += gl_LightSource[0].diffuse*max(0.0,dot(lightVec,norm));
	
	// compute specular component
	vec3 reflectVec = reflect(-lightVec,norm); 	// create reflect view direction
	float spec = pow(clamp(dot(reflectVec,viewVec),0.0,1.0),60.0);
	intensity += gl_LightSource[0].specular*spec;
	
	
	vec4 col = texture2D(colourMap,gl_TexCoord[0].st) * intensity;
	gl_FragColor = vec4(col.rgb,1.0);
}
