// parallax map shader

varying vec3 lightDir;    // interpolated surface local coordinate light direction 
varying vec3 viewDir;     // interpolated surface local coordinate view direction
varying vec3 normal; // normal vector

uniform sampler2D colourMap;
uniform sampler2D normalMap;

uniform float perspectiveBias;
uniform float heightScale;

void main (void)
{
	vec3 lightVec = normalize(lightDir);
	vec3 viewVec = normalize(viewDir);
	vec3 norm = normalize(normal);
	
	vec2 texCoord = vec2(gl_TexCoord[0]);
	float height = (1.0 - texture2D(colourMap,texCoord).a) * heightScale + perspectiveBias;
	texCoord = texCoord - height * viewVec.yx / viewVec.z;

	vec4 intensity = gl_LightSource[0].ambient;
	
	// compute diffuse component
	intensity += gl_LightSource[0].diffuse*max(0.0,dot(lightVec,norm));
	
	// compute specular component
	vec3 reflectVec = reflect(-lightVec,norm); 	// create reflect view direction
	float spec = pow(clamp(dot(reflectVec,viewVec),0.0,1.0),60.0);
	intensity += gl_LightSource[0].specular*spec;
	
	
	vec4 col = texture2D(colourMap,texCoord) * intensity;
	//col = vec4(viewVec,1.0);
	gl_FragColor = vec4(col.rgb,.0);
}
