// Taken primarily from Randi Rost's implementation (c) 3DLabs

varying vec3 lightDir; 	  // interpolated surface local coordinate light direction 
varying vec3 viewDir;     // interpolated surface local coordinate view direction
varying vec2 parallaxDir; 
varying vec2 shadowDir;

attribute vec3 Tangent;
attribute vec3 Binormal;

uniform float perspectiveBias;
uniform float heightScale;

void main (void)
{

	vec3 v; //used as temporary variable
	vec3 lightVec = vec3(gl_ModelViewMatrix  * gl_LightSource[0].position);
	 
	// Transform the input vectors
	vec3 t = normalize(gl_NormalMatrix * Tangent);
   	vec3 b = normalize(gl_NormalMatrix * Binormal);
   	vec3 n = normalize(gl_NormalMatrix * gl_Normal);

   	// Transform light position into surface local coordinates
	vec3 pos = vec3(gl_ModelViewMatrix * gl_Vertex);
	lightVec = lightVec - pos;
   	v.x = dot(lightVec, t);
   	v.y = dot(lightVec, b);
	v.z = dot(lightVec, n);
	lightDir = normalize(v);
	
	// Work out parallax shadow vector
	shadowDir =normalize(lightDir.yx);
	float parallaxLength = -sqrt(1.0 - lightDir.z* lightDir.z) / lightDir.z;
	float parallaxBias = -perspectiveBias + (1.0 - perspectiveBias) * (2.0 * lightDir.z - 1.0);
	shadowDir = -shadowDir * parallaxLength * parallaxBias * heightScale;
	
	pos = -pos;
	v.x = dot(pos, t);
	v.y = dot(pos, b);
	v.z = dot(pos, n);
	viewDir = normalize(v);
	
	// Work out parallax offset vector
	parallaxDir =normalize(viewDir.yx);
	parallaxLength = -sqrt(1.0 - viewDir.z* viewDir.z) / viewDir.z;
	parallaxBias = -perspectiveBias;
	parallaxDir = -parallaxDir * parallaxLength * parallaxBias * heightScale;
	
	
	// cause we have to...
	 gl_TexCoord[0] = gl_MultiTexCoord0;
	 gl_Position = ftransform();
	
}
