// parallax map shader

varying vec3 lightDir;    // interpolated surface local coordinate light direction 
varying vec3 viewDir;     // interpolated surface local coordinate view direction
varying vec2 parallaxDir;
varying vec2 shadowDir;

uniform sampler2D colourMap;
uniform sampler2D normalMap;

uniform float perspectiveBias;
uniform float heightScale;

const float MaxIterations = 18.0;
const float iterStep = 1.0/MaxIterations;
const vec2 step = vec2(2.0,2.0);

void main (void)
{
	vec3 lightVec = normalize(lightDir);
	vec3 viewVec = normalize(viewDir);
	
	vec2 texCoord = gl_TexCoord[0].st;
	
	float h0 = (1.0 - texture2D(colourMap,texCoord).a);
	float h1;
	
	float iter;
   	for (iter = 0.0; iter < 1.0; iter+=iterStep)
   	{
		texCoord -= parallaxDir*step;
		h1 = (1.0 - texture2D(colourMap,texCoord).a);
		if(h1 < iter){
			float frac = clamp((iter-h0)/(h1-h0),0.0,1.0);
			iter += frac; // used in shadowing
			texCoord -= parallaxDir*step*frac;
			break;
		}
		h0 = h1;
	}
	
	float shadow = 1.0;
	vec2 lTexoord = texCoord;
   	for (iter = h0; iter > 0.0; iter-=iterStep)
   	{
		lTexoord -= shadowDir*step;
		h1 = (1.0 - texture2D(colourMap, lTexoord).a);
		if(h0 > iter && h1 > iter){
			shadow -= 0.1;
			//break;
		}
		h0 = h1;
	}
	shadow = max(0.0,shadow);

	vec4 intensity = gl_LightSource[0].ambient;
	// retrieve normal vector from normal map
	vec3 norm = normalize(texture2D(normalMap, texCoord).xyz * 2.0 - 1.0);
	
	// compute diffuse component
	intensity += gl_LightSource[0].diffuse*max(0.0,dot(lightVec,norm))*shadow;
	
	// compute specular component
	vec3 reflectVec = reflect(-lightVec,norm); 	// create reflect view direction
	float spec = pow(clamp(dot(reflectVec,viewVec),0.0,1.0),60.0);
	intensity += gl_LightSource[0].specular*spec*shadow;
	
	
	vec4 col = texture2D(colourMap,texCoord) * intensity;
	//col = vec4(0.0,0.0, iter / MaxIterations,1.0);
	//col = vec4(0.0,0.0, shadow,1.0);
	//col = vec4(shadowDir,0.0,1.0);
	gl_FragColor = vec4(col.rgb,.0);
}
