#include <stdio.h>
#include <math.h>
#include "glincludes.h"
#include "Trackball.h"
#include "shaderdev.h"
#include "ParticleSystem.h"

#define TIME_STEP 10
#define PI 3.1415
#define MAX_FPS 100.0

// Burkhard's trackball at work again
CTrackball trackball;
float fov = 45.0;
float aRatio = 1;


CFAbsoluteTime startTime = CFAbsoluteTimeGetCurrent();
CFAbsoluteTime oldTime = startTime;
int framecount = 0;
float currentFPS = 0;

// Light rotation
#define LIGHT_DIST 4.0
bool lightRot = false;
float lightRotPos = 0.0;
const float ltSpecular[4] = {1.0f,1.0f,1.0f,1.0f};
const float ltAmbient[4] = {0.2f,0.2f,0.2f,1.0f};
const float ltDiffuse[4] = {0.8f,0.8f,0.8f,1.0f};
float ltPosition[4] = {LIGHT_DIST,0.0f,5.0f,1.0f};

//Particle System
ParticleSystem pSystem;
bool animatePS = true;

/* Copied from 372A2 (2002)*/
void handleMouseMotion(int x, int y)
{	
	trackball.tbMotion(x, y);
}

/* Copied from 372A2 (2002)*/
void handleMouseClick(int button, int state, int x, int y)
{
	trackball.tbMouse(button, state, x, y);
}

/* Copied from 372A2 (2002)*/
void handleKeyboardEvent(unsigned char key, int x, int y)
{
	switch(key){
		case 'x':
		case 'y':
		case 'z':
			trackball.tbKeyboard(key);
			break;
		case '0':
			selectShader(0);
			break;
		case '1':
			selectShader(1);
			break;
		case '2':
			selectShader(2);
			break;
		case '3':
			selectShader(3);
			break;
		case '4':
			selectShader(4);
			break;
		case '5':
			selectShader(5);
			break;
		case '6':
			selectShader(6);
			break;
		case '7':
			selectShader(7);
			break;
		case '8':
			selectShader(8);
			break;
		case '9':
			selectShader(9);
			break;
		case 'q':
			pSystem.setDrawMethod(1);
			break;
		case 'w':
			pSystem.setDrawMethod(2);
			break;
		case 'e':
			pSystem.setDrawMethod(3);
			break;
		case 'r':
		case 'R':
			loadShaders();
			break;
		case '-':
			fov += 5;
			if(fov > 150) fov = 150;
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(fov, aRatio, 5.0, 55.0);
			break;
		case '=':
			fov -= 5;
			if(fov < 0) fov = 0;
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(fov, aRatio, 5.0, 55.0);
			break;
		case 'l':
			lightRot = !lightRot;
			break;
		case 'a':
			animatePS = !animatePS;
			break;
		default:
			break;
	}
	// remember to draw new position
	glutPostRedisplay();
	
}

void drawFPS(){
	CFAbsoluteTime newTime = CFAbsoluteTimeGetCurrent();
		if(newTime-oldTime > 1.0){
			currentFPS = framecount/((double)newTime-(double)oldTime);
			oldTime = newTime;
			framecount=0;
		}
		else framecount++;

	glDisable(GL_DEPTH_TEST);
	glMatrixMode( GL_PROJECTION );
    glPushMatrix();
    glLoadIdentity();
    glOrtho( 0.0, 200, 200, 0.0, -10, 10 );
	glMatrixMode( GL_MODELVIEW );
    glPushMatrix();
    glLoadIdentity();
	
	glColor3f(1,1,1);
	glBegin(GL_LINE_LOOP);
	glVertex2f(10,10);
	glVertex2f(20,10);
	glVertex2f(20,190);
	glVertex2f(10,190);
	glEnd();
	
	int len = (currentFPS/MAX_FPS)*178;
	if(len > 178) len = 178;
	
	glColor3f(1,0,0);
	glBegin(GL_TRIANGLE_STRIP);
	glVertex2f(11,189 - len);
	glVertex2f(19,189 - len);
	glVertex2f(11,189);
	glVertex2f(19,189);
	glEnd();
	
    // return to the original projection
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
	glEnable(GL_DEPTH_TEST);
}

void display(void)
{
	// clear all pixels in frame buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	// set up matrix and affect changes made via the trackball
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();	
	gluLookAt(0,0,20, 0,0,0, 0,1,0);
    trackball.tbMatrix();

	glEnable(GL_DEPTH_TEST);
	
	//enable lighting
	glEnable(GL_LIGHTING);
	
	drawShader();
	
	pSystem.draw();

	glDisable(GL_LIGHTING);
	
	glPointSize(10.0);
	glColor3f(1,1,0);
	glBegin(GL_POINTS);
	glVertex3fv((GLfloat*)&ltPosition);
	glEnd();
	
	drawFPS();
	

	glFlush ();
	glutSwapBuffers();
}

/* Copied from 372A2 (2002)*/
void reshape(int width, int height ) {
	glViewport(0, 0, width, height);  // Largest possible square
	trackball.tbReshape(width, height);
	
	aRatio = (GLfloat) width/(GLfloat) height;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(fov, aRatio, 5.0, 55.0);
}

void animate(int value)
{

	if(lightRot){
		lightRotPos = (lightRotPos+0.01);
		ltPosition[0] = cos(lightRotPos)*LIGHT_DIST;
		ltPosition[1] = sin(lightRotPos)*LIGHT_DIST;
		glPushMatrix();
		glLoadIdentity();
		glLightfv(GL_LIGHT0, GL_POSITION, ltPosition);
		glPopMatrix();
	}
	
	if(animatePS){
		pSystem.update(TIME_STEP);
	}

	//Repaint
	glutPostRedisplay();
	// Start the timer for the next iteration of animate
	glutTimerFunc(TIME_STEP,animate,value+1);
}


void init (int width, int height) 
{
	loadResources();
	// select clearing color (for glClear)
	glClearColor (0.0, 0.0, 0.0, 0.0);

	// initialize view (simple orthographic projection)
	aRatio = (GLfloat) width/(GLfloat) height;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(fov, aRatio, 5.0, 55.0);
	trackball.tbInit(GLUT_LEFT_BUTTON);

	// initialize lighting
	glLightfv(GL_LIGHT0, GL_SPECULAR, ltSpecular);
	glLightfv(GL_LIGHT0, GL_AMBIENT, ltAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, ltDiffuse);
	glLightfv(GL_LIGHT0, GL_POSITION, ltPosition);
	
	glEnable(GL_LIGHT0);
	
	//initalize particle system
	pSystem.init();
	
	glutTimerFunc(TIME_STEP,animate,0);

}

int main (int argc, char * argv[]) {
	const int width = 800;
	const int height = 800;
    glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(width, height);
	glutCreateWindow("GLSL-TestApp");
	  
	GLenum err = glewInit();
	if (GLEW_OK != err){
		printf("Error initalizing GLEW\n");
	}
	
    init(width,height);
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutMotionFunc(handleMouseMotion);		// Set function to handle mouse motion
    glutKeyboardFunc(handleKeyboardEvent);
    glutDisplayFunc(display);
    glutReshapeFunc(reshape);
	
	
    glutMainLoop();
	
    return 0;
}
