// __________________________________________________________________________________________________
//    GLGooey Graphical User Interface for OpenGL
//    Copyright (c) 2004 Niel Waldren
//
// This software is provided 'as-is', without any express or implied warranty. In no event will
// the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose, including commercial
// applications, and to alter it and redistribute it freely, subject to the following restrictions:
//
//     1. The origin of this software must not be misrepresented; you must not claim that you
//        wrote the original software. If you use this software in a product, an acknowledgment
//        in the product documentation would be appreciated but is not required.
//
//     2. Altered source versions must be plainly marked as such, and must not be misrepresented
//        as being the original software.
//
//     3. This notice may not be removed or altered from any source distribution.
//
// __________________________________________________________________________________________________
#ifndef __GOOEY_PNG_IMAGE_H__
#define __GOOEY_PNG_IMAGE_H__

#include <string>


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//! \brief PNG image loaded using libpng - This is an example of an image service
class PNGImage
{
    // ----------------------------------------------------------------------------------------------
    //  Xstruction
    // ----------------------------------------------------------------------------------------------
public:
    //! Construction requires the name
    PNGImage(const std::string& aFileName);

    //! Destructor
    virtual ~PNGImage();



    // ---------------------------------------------------------------------------------------------
    //  Public Interface
    // ---------------------------------------------------------------------------------------------
public:
    // Commented in the base class
    virtual int width() const { return width_; }
    virtual int height() const { return height_; }
    virtual int numberOfBitsPerPixel() const { return numberOfBitsPerPixel_; }
    virtual const unsigned char* data() const { return data_; }
    virtual unsigned char* data() { return data_; }
    virtual void release(){ delete[] data_; data_ = NULL;}

    // ---------------------------------------------------------------------------------------------
    //  Implementation
    // ----------------------------------------------------------------------------------------------
private:
    int width_;             //!< The width and height of the image in pixels
    int height_;             //!< The width and height of the image in pixels
    int numberOfBitsPerPixel_; //!< The number of bits used to represent a pixel
    unsigned char* data_;         //!< Pointer to the raw data that represents the image
};
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<



#endif
