/*
 *  shaderdev.cpp
 *  GLUT-GLSL
 *
 *  Created by Serato on 10/09/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "shaderdev.h"
#include "glincludes.h"
#include "png/PNGImage.h"
#include "shader/Shader.h"

#include <stdio.h>

#define normalmap1filename "images/1-normalmap.png"
#define colourmap1filename "images/1-heightmap.png"
#define normalmap2filename "images/2-normalmap.png"
#define colourmap2filename "images/2-heightmap.png"

#define shader1vertfilename "shaders/shader1.vert"
#define shader1fragfilename "shaders/shader1.frag"
#define shader2vertfilename "shaders/shader2.vert"
#define shader2fragfilename "shaders/shader2.frag"
#define shader3vertfilename "shaders/shader3.vert"
#define shader3fragfilename "shaders/shader3.frag"
#define shader4vertfilename "shaders/shader4.vert"
#define shader4fragfilename "shaders/shader4.frag"
#define shader5vertfilename "shaders/shader5.vert"
#define shader5fragfilename "shaders/shader5.frag"
#define shader6vertfilename "shaders/shader6.vert"
#define shader6fragfilename "shaders/shader6.frag"
#define shader7vertfilename "shaders/shader7.vert"
#define shader7fragfilename "shaders/shader7.frag"
#define shader8vertfilename "shaders/shader8.vert"
#define shader8fragfilename "shaders/shader8.frag"

bool initialized = false;

#define HEIGHT_SCALE 0
#define PERSPECTIVE_BIAS 1

// texture references
int textures[2][2];
float textures_param[2][2];

// Shaders
Shader shaders[8];

//currentselection
int selectedTex		= 0;
int selectedShader	= 0;

int registerImage(const char* filename){

	PNGImage anImage = PNGImage(filename);
	GLuint glName = 0;
    glGenTextures(1, &glName);

    if(glName != 0)
    {
        glBindTexture(GL_TEXTURE_2D, glName);

        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

        if(!( (anImage.numberOfBitsPerPixel() == 24) || (anImage.numberOfBitsPerPixel() == 32)) )
			return -1;
        const GLuint format = (anImage.numberOfBitsPerPixel() == 24) ? GL_RGB : GL_RGBA;

        glTexImage2D(GL_TEXTURE_2D, 0, format, anImage.width(), anImage.height(), 0, format, GL_UNSIGNED_BYTE, anImage.data());
		GLenum err = glGetError();
		if(err!=GL_NO_ERROR) 
			printf("glError in registerImage: %s caught\n", (char *)gluErrorString(err));
		
    }
	
	return glName;
}

void loadResources(){
	textures[0][0] = registerImage(colourmap1filename);
	textures[0][1] = registerImage(normalmap1filename);
	textures[1][0] = registerImage(colourmap2filename);
	textures[1][1] = registerImage(normalmap2filename);
	
	textures_param[0][HEIGHT_SCALE] = 0.04;
	textures_param[0][PERSPECTIVE_BIAS] = -0.02;
	textures_param[1][HEIGHT_SCALE] = 0.1;
	textures_param[1][PERSPECTIVE_BIAS] = -0.1;
	
	loadShaders();
	initialized = true;
}

void loadShaders(){
	
	shaders[0].loadVertexShaderFromFile  (shader1vertfilename);
	shaders[0].loadFragmentShaderFromFile(shader1fragfilename);
	shaders[0].compileAndLink();
	shaders[1].loadVertexShaderFromFile  (shader2vertfilename);
	shaders[1].loadFragmentShaderFromFile(shader2fragfilename);
	shaders[1].compileAndLink();
	shaders[2].loadVertexShaderFromFile  (shader3vertfilename);
	shaders[2].loadFragmentShaderFromFile(shader3fragfilename);
	shaders[2].compileAndLink();
	shaders[3].loadVertexShaderFromFile  (shader4vertfilename);
	shaders[3].loadFragmentShaderFromFile(shader4fragfilename);
	shaders[3].compileAndLink();
	shaders[4].loadVertexShaderFromFile  (shader5vertfilename);
	shaders[4].loadFragmentShaderFromFile(shader5fragfilename);
	shaders[4].compileAndLink();
	shaders[5].loadVertexShaderFromFile  (shader6vertfilename);
	shaders[5].loadFragmentShaderFromFile(shader6fragfilename);
	shaders[5].compileAndLink();
	shaders[6].loadVertexShaderFromFile  (shader7vertfilename);
	shaders[6].loadFragmentShaderFromFile(shader7fragfilename);
	shaders[6].compileAndLink();
	shaders[7].loadVertexShaderFromFile  (shader8vertfilename);
	shaders[7].loadFragmentShaderFromFile(shader8fragfilename);
	shaders[7].compileAndLink();
	
	
	updateSelectedShader();
	
	if(!initialized) return;
}

void updateSelectedShader(){
	if(!initialized) return;
	
	int location =  shaders[selectedShader].getVariableLocation("colourMap");
	if(location >= 0)
		shaders[selectedShader].setIntVariable(location,0);
	location =  shaders[selectedShader].getVariableLocation("normalMap");
	if(location >= 0)
		shaders[selectedShader].setIntVariable(location,1);
		
	location =  shaders[selectedShader].getVariableLocation("perspectiveBias");
	if(location >= 0)
		shaders[selectedShader].setFloatVariable(location,textures_param[selectedTex][PERSPECTIVE_BIAS]);
		
	location =  shaders[selectedShader].getVariableLocation("heightScale");
	if(location >= 0)
		shaders[selectedShader].setFloatVariable(location,textures_param[selectedTex][HEIGHT_SCALE]);


}

void selectShader(int shader){
	switch(shader){
		case 1:
			selectedShader = 0;
			break;
		case 2:
			selectedShader = 1;
			break;
		case 3:
			selectedShader = 2;
			break;
		case 4:
			selectedShader = 3;
			break;
		case 5:
			selectedShader = 4;
			break;
		case 6:
			selectedShader = 5;
			break;
		case 7:
			selectedShader = 6;
			break;
		case 8:
			selectedShader = 7;
			break;
		case 9:
			selectedTex = 0;
			break;
		case 0:
			selectedTex = 1;
			break;
	}
	updateSelectedShader();
}


void drawShader(){	
	if(!initialized) return;
	
	// draw scene
	glActiveTexture(GL_TEXTURE0);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textures[selectedTex][0]);
	glActiveTexture(GL_TEXTURE1);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textures[selectedTex][1]);
	
	shaders[selectedShader].enableShader();
	
	glColor3f(1,1,1);
	glVertexAttrib3f(glGetAttribLocationARB(shaders[selectedShader].getHandle(), "Tangent"), 0.0, 1.0, 0.0);
	glVertexAttrib3f(glGetAttribLocationARB(shaders[selectedShader].getHandle(), "Binormal"), 1.0, 0.0, 0.0);
	glNormal3f(0.0, 0.0, 1.0);

	glBegin(GL_TRIANGLE_STRIP);
	glTexCoord2f(0,0);
	glVertex3f(-5,-5,0);
	glTexCoord2f(1,0);
	glVertex3f(5,-5,0);
	glTexCoord2f(0,1);
	glVertex3f(-5,5,0);
	glTexCoord2f(1,1);
	glVertex3f(5,5,0);
	glEnd();
	
	
	shaders[selectedShader].disableShader();
	
	
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE0);
	glDisable(GL_TEXTURE_2D);

}